@@ -1,10 +1,11 @@
 #include "../AdornedRulerPanel.h"
 #include "../Clipboard.h"
 #include "../CommonCommandFlags.h"
-#include "../LabelTrack.h"
+#include "LabelTrack.h"
 #include "../MenuCreator.h"
 #include "NoteTrack.h"
 #include "Project.h"
+#include "ProjectFileManager.h"
 #include "ProjectHistory.h"
 #include "ProjectRate.h"
 #include "ProjectTimeSignature.h"
@@ -12,12 +13,14 @@
 #include "../ProjectWindows.h"
 #include "../SelectUtilities.h"
 #include "SyncLock.h"
+#include "TempoChange.h"
 #include "../TrackPanel.h"
 #include "TrackFocus.h"
 #include "UndoManager.h"
 #include "ViewInfo.h"
 #include "WaveTrack.h"
 #include "WaveTrackUtilities.h"
+#include "TimeStretching.h"
 #include "WaveClip.h"
 #include "SampleBlock.h"
 #include "CommandContext.h"
@@ -33,6 +36,7 @@
 #include "UserException.h"
 #include "Viewport.h"
 
+#include <wx/clipbrd.h>
 #include <wx/frame.h>
 
 // private helper classes and functions
@@ -73,7 +77,7 @@ bool DoPasteText(AudacityProject &project)
    //Presumably, there might be not more than one track
    //that expects text input
    for (auto wt : tracks.Any<WaveTrack>()) {
-      auto& view = WaveChannelView::Get(*wt);
+      auto& view = WaveChannelView::GetFirst(*wt);
       if (view.PasteText(project)) {
          auto &trackPanel = TrackPanel::Get(project);
          trackPanel.Refresh(false);
@@ -88,7 +92,8 @@ wxULongLong EstimateCopyBytesCount(const TrackList& src, const TrackList& dst)
 {
    wxULongLong result{};
    for (auto waveTrack : src.Any<const WaveTrack>()) {
-      const auto samplesCount = waveTrack->GetSequenceSamplesCount();
+      const auto samplesCount =
+         WaveTrackUtilities::GetSequenceSamplesCount(*waveTrack);
       result += samplesCount.as_long_long() *
          SAMPLE_SIZE(waveTrack->GetSampleFormat());
    }
@@ -99,20 +104,19 @@ BlockArray::size_type EstimateCopiedBlocks(const TrackList& src, const TrackList
 {
    BlockArray::size_type result{};
    for (const auto waveTrack : src.Any<const WaveTrack>())
-      result += waveTrack->CountBlocks();
+      result += WaveTrackUtilities::CountBlocks(*waveTrack);
    return result;
 }
 
 std::shared_ptr<TrackList> DuplicateDiscardTrimmed(const TrackList& src) {
    auto result = TrackList::Create(nullptr);
    for (auto track : src) {
-      const auto copies =
+      const auto pTrack =
          track->Copy(track->GetStartTime(), track->GetEndTime(), false);
-      const auto pTrack = *copies->begin();
       pTrack->MoveTo(track->GetStartTime());
-      if (const auto waveTrack = dynamic_cast<WaveTrack*>(pTrack))
-         waveTrack->DiscardTrimmed();
-      result->Append(std::move(*copies));
+      if (const auto waveTrack = dynamic_cast<WaveTrack*>(pTrack.get()))
+         WaveTrackUtilities::DiscardTrimmed(*waveTrack);
+      result->Add(pTrack);
    }
    return result;
 }
@@ -143,7 +147,7 @@ void DoPasteNothingSelected(AudacityProject &project, const TrackList& src, doub
    const double projRate = ProjectRate::Get( project ).GetRate();
    const double projTempo = ProjectTimeSignature::Get(project).GetTempo();
    const double srcTempo =
-      pFirstNewTrack ? pFirstNewTrack->GetProjectTempo().value_or(projTempo) :
+      pFirstNewTrack ? GetProjectTempo(*pFirstNewTrack).value_or(projTempo) :
                        projTempo;
    // Apply adequate stretching to the selection. A selection of 10 seconds of
    // audio in project A should become 5 seconds in project B if tempo in B is
@@ -168,7 +172,9 @@ bool HasHiddenData(const TrackList& trackList)
 {
    const auto range = trackList.Any<const WaveTrack>();
    return std::any_of(range.begin(), range.end(),
-      [](const WaveTrack *pTrack){ return pTrack->HasHiddenData(); });
+      [](const WaveTrack *pTrack){
+         return WaveTrackUtilities::HasHiddenData(*pTrack);
+   });
 }
 
 // Menu handler functions
@@ -230,8 +236,24 @@ void OnRedo(const CommandContext &context)
       Viewport::Get(project).ShowTrack(*t);
 }
 
+void ClearClipboard()
+{
+   if (wxTheClipboard->Open())
+   {
+      // Clear the data from the clipboard, so that the next paste doesn't
+      // attempt to import whatever the user copied from the OS in a prior
+      // action.
+      const auto success = wxTheClipboard->SetData(safenew wxTextDataObject);
+      assert(success);
+      wxTheClipboard->Clear();
+      wxTheClipboard->Close();
+   }
+}
+
 void OnCut(const CommandContext &context)
 {
+   ClearClipboard();
+
    auto &project = context.project;
    auto &tracks = TrackList::Get( project );
    auto &trackPanel = TrackPanel::Get( project );
@@ -253,7 +275,7 @@ void OnCut(const CommandContext &context)
    //Presumably, there might be not more than one track
    //that expects text input
    for (auto wt : tracks.Any<WaveTrack>()) {
-      auto& view = WaveChannelView::Get(*wt);
+      auto& view = WaveChannelView::GetFirst(*wt);
       if (view.CutSelectedText(context.project)) {
          trackPanel.Refresh(false);
          return;
@@ -271,13 +293,13 @@ void OnCut(const CommandContext &context)
       [&](NoteTrack &n) {
          // Since portsmf has a built-in cut operator, we use that instead
          auto dest = n.Cut(selectedRegion.t0(), selectedRegion.t1());
-         newClipboard.Append(std::move(*dest));
+         newClipboard.Add(dest);
       },
 #endif
       [&](Track &n) {
          if (n.SupportsBasicEditing()) {
             auto dest = n.Copy(selectedRegion.t0(), selectedRegion.t1());
-            newClipboard.Append(std::move(*dest));
+            newClipboard.Add(dest);
          }
       }
    );
@@ -293,7 +315,7 @@ void OnCut(const CommandContext &context)
    // Proceed to change the project.  If this throws, the project will be
    // rolled back by the top level handler.
 
-   (tracks.Any() + &SyncLock::IsSelectedOrSyncLockSelected).Visit(
+   (tracks.Any() + &SyncLock::IsSelectedOrSyncLockSelectedP).Visit(
 #if defined(USE_MIDI)
       [](NoteTrack&) {
          //if NoteTrack, it was cut, so do not clear anything
@@ -332,7 +354,7 @@ void OnDelete(const CommandContext &context)
    for (auto n : tracks) {
       if (!n->SupportsBasicEditing())
          continue;
-      if (SyncLock::IsSelectedOrSyncLockSelected(n)) {
+      if (SyncLock::IsSelectedOrSyncLockSelected(*n)) {
          n->Clear(selectedRegion.t0(), selectedRegion.t1());
       }
    }
@@ -350,6 +372,8 @@ void OnDelete(const CommandContext &context)
 
 void OnCopy(const CommandContext &context)
 {
+   ClearClipboard();
+
    auto &project = context.project;
    auto &tracks = TrackList::Get( project );
    auto &trackPanel = TrackPanel::Get( project );
@@ -365,7 +389,7 @@ void OnCopy(const CommandContext &context)
    //Presumably, there might be not more than one track
    //that expects text input
    for (auto wt : tracks.Any<WaveTrack>()) {
-      auto& view = WaveChannelView::Get(*wt);
+      auto& view = WaveChannelView::GetFirst(*wt);
       if (view.CopySelectedText(context.project)) {
          return;
       }
@@ -380,7 +404,7 @@ void OnCopy(const CommandContext &context)
    for (auto n : tracks.Selected()) {
       if (n->SupportsBasicEditing()) {
          auto dest = n->Copy(selectedRegion.t0(), selectedRegion.t1());
-         newClipboard.Append(std::move(*dest));
+         newClipboard.Add(dest);
       }
    }
 
@@ -423,7 +447,9 @@ std::shared_ptr<const TrackList> FindSourceTracks(const CommandContext &context)
    auto discardTrimmed = false;
    if (&context.project != &*clipboard.Project().lock()) {
       const auto waveClipCopyPolicy = TracksBehaviorsAudioTrackPastePolicy.Read();
-      if(waveClipCopyPolicy == wxT("Ask") && HasHiddenData(clipboard.GetTracks())) {
+      if (waveClipCopyPolicy == wxT("Ask") &&
+         HasHiddenData(clipboard.GetTracks()))
+      {
          AudioPasteDialog audioPasteDialog(
             &window,
             EstimateCopyBytesCount(clipboard.GetTracks(), tracks)
@@ -481,7 +507,7 @@ bool FitsInto(const Track &src, const Track &dst)
    // Mono can "fit" into stereo, by duplication of the channel
    // Otherwise non-wave tracks always have just one "channel"
    // Future:  Fit stereo into mono too, using mix-down
-   return TrackList::NChannels(src) <= TrackList::NChannels(dst);
+   return src.NChannels() <= dst.NChannels();
 }
 
 // First, destination track; second, source
@@ -519,6 +545,18 @@ Correspondence FindCorrespondence(
 
 void OnPaste(const CommandContext &context)
 {
+   if (wxTheClipboard->IsSupported(wxDF_FILENAME) && wxTheClipboard->Open())
+   {
+      wxFileDataObject data;
+      const auto hadData = wxTheClipboard->GetData(data);
+      wxTheClipboard->Close();
+      if (hadData)
+      {
+         ProjectFileManager::Get(context.project).Import(data.GetFilenames());
+         return;
+      }
+   }
+
    auto &project = context.project;
 
    // Handle text paste first.
@@ -580,7 +618,7 @@ void OnPaste(const CommandContext &context)
       if (iPair == endPair)
          // Nothing more to paste
          break;
-      auto group = SyncLock::Group(*range.first);
+      auto group = SyncLock::Group(**range.first);
       next = tracks.Find(*group.rbegin());
       ++next;
 
@@ -588,13 +626,13 @@ void OnPaste(const CommandContext &context)
          // Nothing to paste into this group
          continue;
 
-      // Inner loop over the group by tracks (not channels)
-      for (auto leader : group) {
-         if (iPair == endPair || leader != iPair->first) {
+      // Inner loop over the sync-lock group by tracks
+      for (auto member : group) {
+         if (iPair == endPair || member != iPair->first) {
             if (isSyncLocked) {
                // Track is not pasted into but must be adjusted
-               if (t1 != newT1 && t1 <= leader->GetEndTime()) {
-                  leader->SyncLockAdjust(t1, newT1);
+               if (t1 != newT1 && t1 <= member->GetEndTime()) {
+                  member->SyncLockAdjust(t1, newT1);
                   bPastedSomething = true;
                }
             }
@@ -602,10 +640,10 @@ void OnPaste(const CommandContext &context)
          else {
             // Remember first pasted-into track, to focus it
             if (!ff)
-               ff = leader;
+               ff = member;
             // Do the pasting!
             const auto src = (iPair++)->second;
-            leader->TypeSwitch(
+            member->TypeSwitch(
                [&](WaveTrack &wn){
                   bPastedSomething = true;
                   // For correct remapping of preserved split lines:
@@ -621,10 +659,24 @@ void OnPaste(const CommandContext &context)
                   const auto merge = newClipOnPaste ? false : true;
                   const auto preserveExistingBoundaries = newClipOnPaste ? false : true;
                   auto clearByTrimming = newClipOnPaste ? true : false;
-                  wn.ClearAndPaste(
-                     t0, t1, *static_cast<const WaveTrack*>(src),
-                     preserveExistingBoundaries, merge, &warper,
-                     clearByTrimming);
+                  if(src->NChannels() == 1 && wn.NChannels() == 2)
+                  {
+                     // When the source is mono, may paste its only channel
+                     // repeatedly into a stereo track
+                     const auto pastedTrack = std::static_pointer_cast<WaveTrack>(src->Duplicate());
+                     pastedTrack->MonoToStereo();
+                     wn.ClearAndPaste(
+                        t0, t1, *pastedTrack,
+                        preserveExistingBoundaries, merge, &warper,
+                        clearByTrimming);
+                  }
+                  else
+                  {
+                     wn.ClearAndPaste(
+                        t0, t1, *static_cast<const WaveTrack*>(src),
+                        preserveExistingBoundaries, merge, &warper,
+                        clearByTrimming);
+                  }
                },
                [&](LabelTrack &ln){
                   // Per Bug 293, users expect labels to move on a paste into
@@ -648,6 +700,25 @@ void OnPaste(const CommandContext &context)
    // TODO: What if we clicked past the end of the track?
 
    if (bPastedSomething) {
+
+      if(!isSyncLocked && GetEditClipsCanMove())
+      {
+         //Special case when pasting without sync lock and
+         //"...move other clips" option is 
+         //Also shift all intervals in all other selected tracks that
+         //starts after t0
+         const auto offset = srcTracks->GetEndTime() - (t1 - t0);
+         for(auto track : tracks.Selected<Track>())
+         {
+            const auto it = std::find_if(correspondence.begin(), correspondence.end(),
+                                   [=](auto& p) { return p.first == track; });
+            if(it != correspondence.end())
+               continue;
+
+            track->ShiftBy(t0, offset);
+         }
+      }
+
       ViewInfo::Get(project).selectedRegion
          .setTimes( t0, t0 + clipboard.Duration() );
 
@@ -676,9 +747,8 @@ void OnDuplicate(const CommandContext &context)
 
       // Make copies not for clipboard but for direct addition to the project
       auto dest = n->Copy(selectedRegion.t0(), selectedRegion.t1(), false);
-      (*dest->begin())
-         ->MoveTo(std::max(selectedRegion.t0(), n->GetStartTime()));
-      tracks.Append(std::move(*dest));
+      dest->MoveTo(std::max(selectedRegion.t0(), n->GetStartTime()));
+      tracks.Add(dest);
 
       // This break is really needed, else we loop infinitely
       if (n == last)
@@ -703,14 +773,14 @@ void OnSplitCut(const CommandContext &context)
 
    tracks.Selected().Visit(
       [&](WaveTrack &n) {
-         auto tracks = n.SplitCut(selectedRegion.t0(), selectedRegion.t1());
-         newClipboard.Append(std::move(*tracks));
+         auto track = n.SplitCut(selectedRegion.t0(), selectedRegion.t1());
+         newClipboard.Add(track);
       },
       [&](Track &n) {
          if (n.SupportsBasicEditing()) {
             auto dest = n.Copy(selectedRegion.t0(), selectedRegion.t1());
             n.Silence(selectedRegion.t0(), selectedRegion.t1());
-            newClipboard.Append(std::move(*dest));
+            newClipboard.Add(dest);
          }
       }
    );
@@ -720,7 +790,7 @@ void OnSplitCut(const CommandContext &context)
       selectedRegion.t0(), selectedRegion.t1(), project.shared_from_this());
 
    ProjectHistory::Get(project)
-      .PushState(XO("Split-cut to the clipboard"), XO("Split Cut"));
+      .PushState(XO("Cut to clipboard and leave gap"), XO("Cut and leave gap"));
 }
 
 void OnSplitDelete(const CommandContext &context)
@@ -752,7 +822,7 @@ void OnSilence(const CommandContext &context)
    auto &selectedRegion = ViewInfo::Get(project).selectedRegion;
 
    const auto selectedWaveTracks = tracks.Selected<WaveTrack>();
-   WaveTrackUtilities::WithClipRenderingProgress(
+   TimeStretching::WithClipRenderingProgress(
       [&](const ProgressReporter& parent) {
          BasicUI::SplitProgress(
             selectedWaveTracks.begin(), selectedWaveTracks.end(),
@@ -794,7 +864,7 @@ void OnSplit(const CommandContext &context)
    auto &project = context.project;
    auto &tracks = TrackList::Get(project);
    auto [sel0, sel1] = FindSelection(context);
-   if (auto *pTrack = *tracks.Find(context.temporarySelection.pTrack)) {
+   if (auto *pTrack = context.temporarySelection.pTrack) {
       if (auto pWaveTrack = dynamic_cast<WaveTrack*>(pTrack))
          pWaveTrack->Split(sel0, sel1);
       else
@@ -876,8 +946,8 @@ void OnSplitNew(const CommandContext &context)
             if (dest) {
                // The copy function normally puts the clip at time 0
                // This offset lines it up with the original track's timing
-               (*dest->begin())->MoveTo(newt0);
-               tracks.Append(std::move(*dest));
+               dest->MoveTo(newt0);
+               tracks.Add(dest);
             }
             wt.SplitDelete(newt0, newt1);
          }
@@ -909,7 +979,7 @@ void OnJoin(const CommandContext &context)
    auto &tracks = TrackList::Get(project);
    auto &selectedRegion = ViewInfo::Get(project).selectedRegion;
    const auto selectedTracks = tracks.Selected<WaveTrack>();
-   WaveTrackUtilities::WithClipRenderingProgress(
+   TimeStretching::WithClipRenderingProgress(
       [&](const ProgressReporter& reportProgress) {
          using namespace BasicUI;
          SplitProgress(
@@ -1034,10 +1104,11 @@ const ReservedCommandFlag
          return false;
 
       const auto selectedTracks = TrackList::Get(project).Selected<const WaveTrack>();
-      for(const auto track : selectedTracks)
+      for (const auto track : selectedTracks)
       {
          const auto selectedClips =
-            track->GetClipsIntersecting(viewInfo.selectedRegion.t0(), viewInfo.selectedRegion.t1());
+            WaveTrackUtilities::GetClipsIntersecting(*track,
+               viewInfo.selectedRegion.t0(), viewInfo.selectedRegion.t1());
          if(selectedClips.size() > 1)
             return true;
       }
@@ -1110,11 +1181,11 @@ auto EditMenu()
             Menu( wxT("RemoveSpecial"), XXO("R&emove Special"),
                Section( "",
                   /* i18n-hint: (verb) Do a special kind of cut*/
-                  Command( wxT("SplitCut"), XXO("Spl&it Cut"), OnSplitCut,
+                  Command( wxT("SplitCut"), XXO("&Cut and leave gap"), OnSplitCut,
                      NotBusyTimeAndTracksFlags,
                      Options{ wxT("Ctrl+Alt+X") } ),
                   /* i18n-hint: (verb) Do a special kind of DELETE*/
-                  Command( wxT("SplitDelete"), XXO("Split D&elete"), OnSplitDelete,
+                  Command( wxT("SplitDelete"), XXO("&Delete and leave gap"), OnSplitDelete,
                      NotBusyTimeAndTracksFlags,
                      Options{ wxT("Ctrl+Alt+K") } )
                ),
